@@ -53,20 +53,17 @@ return true;
 this.readSymmetry ();
 return true;
 }if (this.line.indexOf ("Output coordinates in ") >= 0) {
+var thisLine = this.line;
 if (!this.htMOs.isEmpty ()) this.checkMOs ();
 if (!this.doGetModel (++this.modelNumber, null)) return this.checkLastModel ();
 this.equivalentAtomSets++;
-this.readAtoms ();
+this.readAtoms (thisLine);
 return true;
 }if (this.line.indexOf ("Vibrational analysis") >= 0) {
 this.readFrequencies ();
 return true;
 }if (!this.doProcessLines) return true;
-if (this.line.indexOf ("ENERGY GRADIENTS") >= 0) {
-this.equivalentAtomSets++;
-this.readGradients ();
-return true;
-}if (this.line.startsWith ("  Mulliken analysis of the total density")) {
+if (this.line.startsWith ("  Mulliken analysis of the total density")) {
 if (this.equivalentAtomSets != 0) this.readPartialCharges ();
 return true;
 }if (this.line.contains ("Basis \"ao basis\"") && this.doReadMolecularOrbitals) {
@@ -147,8 +144,8 @@ for (var i = this.asc.iSet; --n >= 0 && i >= 0; --i) if (namedSets == null || !n
 
 }, "~S,JU.BS,~N");
 Clazz.defineMethod (c$, "readAtoms", 
- function () {
-var scale = (this.line.indexOf ("angstroms") < 0 ? 0.5291772 : 1);
+ function (thisLine) {
+var scale = (thisLine.indexOf ("angstroms") < 0 ? 0.5291772 : 1);
 this.readLines (3);
 var tokens;
 this.haveEnergy = false;
@@ -167,25 +164,7 @@ this.setEnergy (this.energyKey, this.energyValue);
 this.asc.setAtomSetModelProperty ("Step", "converged");
 } else if (this.inInput) {
 this.asc.setAtomSetName ("Input");
-}});
-Clazz.defineMethod (c$, "readGradients", 
- function () {
-this.readLines (3);
-var tokens;
-this.asc.newAtomSet ();
-if (this.equivalentAtomSets > 1) {
-var p = this.asc.getAtomSetAuxiliaryInfoValue (this.asc.iSet - 1, "modelProperties");
-if (p != null) this.asc.setCurrentModelInfo ("modelProperties", p.clone ());
-}this.asc.setAtomSetModelProperty ("vector", "gradient");
-this.asc.setAtomSetModelProperty (".PATH", "Task " + this.taskNumber + J.adapter.smarter.SmarterJmolAdapter.PATH_SEPARATOR + "Gradients");
-while (this.rd () != null && this.line.length > 0) {
-tokens = this.getTokens ();
-if (tokens.length < 8) break;
-var atom = this.setAtomCoordScaled (null, tokens, 2, 0.5291772);
-atom.atomName = this.fixTag (tokens[1]);
-this.asc.addVibrationVector (atom.index, -this.parseFloatStr (tokens[5]), -this.parseFloatStr (tokens[6]), -this.parseFloatStr (tokens[7]));
-}
-});
+}}, "~S");
 Clazz.defineMethod (c$, "readFrequencies", 
  function () {
 var firstFrequencyAtomSetIndex = this.asc.atomSetCount;
@@ -274,11 +253,11 @@ this.shellCount = 0;
 this.nBasisFunctions = 0;
 var isD6F10 = (this.line.indexOf ("cartesian") >= 0);
 if (isD6F10) {
-this.getDFMap (J.adapter.readers.quantum.NWChemReader.$DC_LIST, 4, "DXX   DYY   DZZ   DXY   DXZ   DYZ", 3);
-this.getDFMap (J.adapter.readers.quantum.NWChemReader.$FC_LIST, 6, "XXX   YYY   ZZZ   XYY   XXY   XXZ   XZZ   YZZ   YYZ   XYZ", 3);
+this.getDFMap ("DC", J.adapter.readers.quantum.NWChemReader.$DC_LIST, 4, "DXX   DYY   DZZ   DXY   DXZ   DYZ", 3);
+this.getDFMap ("FC", J.adapter.readers.quantum.NWChemReader.$FC_LIST, 6, "XXX   YYY   ZZZ   XYY   XXY   XXZ   XZZ   YZZ   YYZ   XYZ", 3);
 } else {
-this.getDFMap (J.adapter.readers.quantum.NWChemReader.$DS_LIST, 3, "d0    d1+   d1-   d2+   d2-", 2);
-this.getDFMap (J.adapter.readers.quantum.NWChemReader.$FS_LIST, 5, "f0    f1+   f1-   f2+   f2-   f3+   f3-", 2);
+this.getDFMap ("DS", J.adapter.readers.quantum.NWChemReader.$DS_LIST, 3, "d0    d1+   d1-   d2+   d2-", 2);
+this.getDFMap ("FS", J.adapter.readers.quantum.NWChemReader.$FS_LIST, 5, "f0    f1+   f1-   f2+   f2-   f3+   f3-", 2);
 }this.shells =  new JU.Lst ();
 var atomInfo =  new java.util.Hashtable ();
 var atomSym = null;
@@ -334,9 +313,9 @@ this.nBasisFunctions += nF;
 break;
 }
 var slater =  Clazz.newIntArray (4, 0);
-slater[0] = i;
+slater[0] = i + 1;
 slater[1] = (isD6F10 ? J.adapter.readers.quantum.BasisFunctionReader.getQuantumShellTagID (type) : J.adapter.readers.quantum.BasisFunctionReader.getQuantumShellTagIDSpherical (type));
-slater[2] = this.gaussianCount;
+slater[2] = this.gaussianCount + 1;
 slater[3] = nGaussians;
 this.shells.addLast (slater);
 for (var ifunc = 0; ifunc < nGaussians; ifunc++) gdata.addLast (shellData.get (ifunc)[1]);
