@@ -1,18 +1,16 @@
 Clazz.declarePackage ("JS");
-Clazz.load (["JU.BS", "$.Lst", "$.V3"], "JS.CIPChirality", ["java.util.Arrays", "$.Hashtable", "JU.Measure", "$.P4", "$.PT", "JU.BSUtil", "$.Elements", "$.Logger", "JV.JC"], function () {
+Clazz.load (["JU.BS", "$.V3"], "JS.CIPChirality", ["java.util.Arrays", "$.Hashtable", "JU.Lst", "$.Measure", "$.P4", "$.PT", "JU.BSUtil", "$.Elements", "$.Logger", "JV.JC"], function () {
 c$ = Clazz.decorateAsClass (function () {
 this.bsNeedRule = null;
 this.ptIDLogger = 0;
 this.root = null;
 this.currentRule = 1;
-this.lstSmallRings = null;
-this.bsAtropisomeric = null;
-this.bsKekuleAmbiguous = null;
 this.bsAzacyclic = null;
 this.vNorm = null;
 this.vNorm2 = null;
 this.vTemp = null;
 this.setAuxiliary = false;
+this.data = null;
 if (!Clazz.isClassDefined ("JS.CIPChirality.CIPAtom")) {
 JS.CIPChirality.$CIPChirality$CIPAtom$ ();
 }
@@ -20,7 +18,6 @@ Clazz.instantialize (this, arguments);
 }, JS, "CIPChirality");
 Clazz.prepareFields (c$, function () {
 this.bsNeedRule =  new JU.BS ();
-this.lstSmallRings =  new JU.Lst ();
 this.vNorm =  new JU.V3 ();
 this.vNorm2 =  new JU.V3 ();
 this.vTemp =  new JU.V3 ();
@@ -36,26 +33,18 @@ function () {
 Clazz.defineMethod (c$, "init", 
  function () {
 this.ptIDLogger = 0;
-this.lstSmallRings.clear ();
-this.bsKekuleAmbiguous = null;
-this.bsAtropisomeric =  new JU.BS ();
 });
 Clazz.defineMethod (c$, "getChiralityForAtoms", 
-function (atoms, bsAtoms, bsAtropisomeric, bsHelixM, bsHelixP, setAuxiliary) {
-if (bsAtoms.isEmpty ()) return;
+function (data) {
+this.data = data;
+if (data.bsAtoms.isEmpty ()) return;
 this.init ();
-this.setAuxiliary = (setAuxiliary && bsAtoms.cardinality () == 1);
-this.bsAtropisomeric = (bsAtropisomeric == null ?  new JU.BS () : bsAtropisomeric);
-var bs = JU.BSUtil.copy (bsAtoms);
-this.lstSmallRings =  new JU.Lst ();
-while (!bs.isEmpty ()) this.getSmallRings (atoms[bs.nextSetBit (0)], bs);
-
-this.bsKekuleAmbiguous = this.getKekule (atoms);
-this.bsAzacyclic = this.getAzacyclic (atoms, bsAtoms);
-var bsToDo = JU.BSUtil.copy (bsAtoms);
-var haveAlkenes = this.preFilterAtomList (atoms, bsToDo);
+this.setAuxiliary = (this.setAuxiliary && data.bsAtoms.cardinality () == 1);
+this.bsAzacyclic = this.getAzacyclic (data.atoms, data.bsAtoms);
+var bsToDo = JU.BSUtil.copy (data.bsAtoms);
+var haveAlkenes = this.preFilterAtomList (data.atoms, bsToDo);
 for (var i = bsToDo.nextSetBit (0); i >= 0; i = bsToDo.nextSetBit (i + 1)) {
-var a = atoms[i];
+var a = data.atoms[i];
 a.setCIPChirality (0);
 this.ptIDLogger = 0;
 var c = this.getAtomChiralityLimited (a, null, null);
@@ -63,26 +52,29 @@ a.setCIPChirality (c == 0 ? 3 : c | ((this.currentRule - 1) << 5));
 }
 if (haveAlkenes) {
 var lstEZ =  new JU.Lst ();
-for (var i = bsToDo.nextSetBit (0); i >= 0; i = bsToDo.nextSetBit (i + 1)) this.getAtomBondChirality (atoms[i], lstEZ, bsToDo);
-
-if (this.lstSmallRings.size () > 0 && lstEZ.size () > 0) this.clearSmallRingEZ (atoms, lstEZ);
-if (bsHelixM != null) for (var i = bsHelixM.nextSetBit (0); i >= 0; i = bsHelixM.nextSetBit (i + 1)) atoms[i].setCIPChirality (17);
-
-if (bsHelixP != null) for (var i = bsHelixP.nextSetBit (0); i >= 0; i = bsHelixP.nextSetBit (i + 1)) atoms[i].setCIPChirality (18);
+for (var i = bsToDo.nextSetBit (0); i >= 0; i = bsToDo.nextSetBit (i + 1)) this.getAtomBondChirality (data.atoms[i], lstEZ, bsToDo);
 
+if (data.lstSmallRings.length > 0 && lstEZ.size () > 0) this.clearSmallRingEZ (data.atoms, lstEZ);
+this.setStereoFromSmiles (data.bsHelixM, 17, data.atoms);
+this.setStereoFromSmiles (data.bsHelixP, 18, data.atoms);
 }if (JU.Logger.debugging) {
-JU.Logger.info ("sp2-aromatic = " + this.bsKekuleAmbiguous);
-JU.Logger.info ("smallRings = " + JU.PT.toJSON (null, this.lstSmallRings));
-}}, "~A,JU.BS,JU.BS,JU.BS,JU.BS,~B");
+JU.Logger.info ("sp2-aromatic = " + data.bsKekuleAmbiguous);
+JU.Logger.info ("smallRings = " + JU.PT.toJSON (null, data.lstSmallRings));
+}}, "JS.CIPData");
+Clazz.defineMethod (c$, "setStereoFromSmiles", 
+ function (bsHelix, stereo, atoms) {
+if (bsHelix != null) for (var i = bsHelix.nextSetBit (0); i >= 0; i = bsHelix.nextSetBit (i + 1)) atoms[i].setCIPChirality (stereo);
+
+}, "JU.BS,~N,~A");
 Clazz.defineMethod (c$, "getAzacyclic", 
  function (atoms, bsAtoms) {
 var bsAza = null;
 for (var i = bsAtoms.nextSetBit (0); i >= 0; i = bsAtoms.nextSetBit (i + 1)) {
 var atom = atoms[i];
-if (atom.getElementNumber () != 7 || atom.getCovalentBondCount () != 3 || this.bsKekuleAmbiguous.get (i)) continue;
+if (atom.getElementNumber () != 7 || atom.getCovalentBondCount () != 3 || this.data.bsKekuleAmbiguous.get (i)) continue;
 var nRings =  new JU.Lst ();
-for (var j = this.lstSmallRings.size (); --j >= 0; ) {
-var bsRing = this.lstSmallRings.get (j);
+for (var j = this.data.lstSmallRings.length; --j >= 0; ) {
+var bsRing = this.data.lstSmallRings[j];
 if (bsRing.get (i)) nRings.addLast (bsRing);
 }
 var nr = nRings.size ();
@@ -199,53 +191,14 @@ function (a) {
 var n = a.getElementNumber ();
 return (n > 2 && n <= 10);
 }, "JU.SimpleNode");
-Clazz.defineMethod (c$, "getKekule", 
- function (atoms) {
-var nRings = this.lstSmallRings.size ();
-var bs =  new JU.BS ();
-var bsDone =  new JU.BS ();
-for (var i = nRings; --i >= 0; ) {
-if (bsDone.get (i)) continue;
-var bsRing = this.lstSmallRings.get (i);
-if (bsRing.cardinality () != 6) {
-bsDone.set (i);
-continue;
-}var nPI = 0;
-for (var j = bsRing.nextSetBit (0); j >= 0; j = bsRing.nextSetBit (j + 1)) {
-var a = atoms[j];
-if (bs.get (a.getIndex ())) {
-nPI++;
-continue;
-}var nb = a.getCovalentBondCount ();
-if (nb == 3 || nb == 2) {
-var bonds = a.getEdges ();
-for (var k = bonds.length; --k >= 0; ) {
-var b = bonds[k];
-if (b.getCovalentOrder () != 2) continue;
-if (bsRing.get (b.getOtherNode (a).getIndex ())) {
-nPI++;
-break;
-}}
-}}
-if (nPI == 6) {
-bs.or (bsRing);
-bsDone.set (i);
-i = nRings;
-}}
-return bs;
-}, "~A");
-Clazz.defineMethod (c$, "getSmallRings", 
- function (atom, bs) {
-(this.root = Clazz.innerTypeInstance (JS.CIPChirality.CIPAtom, this, null).create (atom, null, false, false, false)).addSmallRings (bs);
-}, "JU.SimpleNode,JU.BS");
 Clazz.defineMethod (c$, "clearSmallRingEZ", 
  function (atoms, lstEZ) {
-for (var j = this.lstSmallRings.size (); --j >= 0; ) this.lstSmallRings.get (j).andNot (this.bsAtropisomeric);
+for (var j = this.data.lstSmallRings.length; --j >= 0; ) this.data.lstSmallRings[j].andNot (this.data.bsAtropisomeric);
 
 for (var i = lstEZ.size (); --i >= 0; ) {
 var ab = lstEZ.get (i);
-for (var j = this.lstSmallRings.size (); --j >= 0; ) {
-var ring = this.lstSmallRings.get (j);
+for (var j = this.data.lstSmallRings.length; --j >= 0; ) {
+var ring = this.data.lstSmallRings[j];
 if (ring.get (ab[0]) && ring.get (ab[1])) {
 atoms[ab[0]].setCIPChirality (3);
 atoms[ab[1]].setCIPChirality (3);
@@ -266,7 +219,7 @@ Clazz.defineMethod (c$, "getAtomBondChirality",
 var index = atom.getIndex ();
 var bonds = atom.getEdges ();
 var c = 0;
-var isAtropic = this.bsAtropisomeric.get (index);
+var isAtropic = this.data.bsAtropisomeric.get (index);
 for (var j = bonds.length; --j >= 0; ) {
 var bond = bonds[j];
 var atom1;
@@ -274,7 +227,7 @@ var index1;
 if (isAtropic) {
 atom1 = bonds[j].getOtherNode (atom);
 index1 = atom1.getIndex ();
-if (!this.bsAtropisomeric.get (index1)) continue;
+if (!this.data.bsAtropisomeric.get (index1)) continue;
 c = this.setBondChirality (atom, atom1, atom, atom1, true);
 } else if (bond.getCovalentOrder () == 2) {
 atom1 = this.getLastCumuleneAtom (bond, atom, null, null);
@@ -320,6 +273,8 @@ break;
 Clazz.defineMethod (c$, "getAtomChiralityLimited", 
  function (atom, cipAtom, parentAtom) {
 var rs = 0;
+this.bsNeedRule.clearAll ();
+this.bsNeedRule.set (1);
 try {
 var isAlkeneEndCheck = (atom == null);
 if (isAlkeneEndCheck) {
@@ -328,9 +283,7 @@ cipAtom.htPathPoints = (cipAtom.parent = Clazz.innerTypeInstance (JS.CIPChiralit
 } else {
 if (!(this.root = cipAtom = Clazz.innerTypeInstance (JS.CIPChirality.CIPAtom, this, null).create (atom, null, false, false, false)).isSP3) {
 return 0;
-}this.bsNeedRule.clearAll ();
-this.bsNeedRule.set (1);
-}if (cipAtom.setNode ()) {
+}}if (cipAtom.setNode ()) {
 for (this.currentRule = 1; this.currentRule <= 9; this.currentRule++) {
 if (JU.Logger.debugging) JU.Logger.info ("-Rule " + this.getRuleName (this.currentRule) + " CIPChirality for " + cipAtom + "-----");
 switch (this.currentRule) {
@@ -395,7 +348,7 @@ var b2 = Clazz.innerTypeInstance (JS.CIPChirality.CIPAtom, this, null).create (b
 var btop = this.getAtomChiralityLimited (null, b2, pb) - 1;
 var ruleB = this.currentRule;
 var c = (atop >= 0 && btop >= 0 ? this.getEneChirality (b2.atoms[btop], b2, a1, a1.atoms[atop], isAxial, true) : 0);
-if (c != 0 && (isAxial || !this.bsAtropisomeric.get (a.getIndex ()) && !this.bsAtropisomeric.get (b.getIndex ()))) {
+if (c != 0 && (isAxial || !this.data.bsAtropisomeric.get (a.getIndex ()) && !this.data.bsAtropisomeric.get (b.getIndex ()))) {
 if (isAxial && ((ruleA >= 8) != (ruleB >= 8))) {
 c |= 8;
 }a.setCIPChirality (c | ((ruleA - 1) << 5));
@@ -526,7 +479,8 @@ this.isAlkene = c;
 this.atom = a;
 this.atomIndex = a.getIndex ();
 if (a.getIsotopeNumber () > 0) this.b$["JS.CIPChirality"].bsNeedRule.set (3);
-this.isKekuleAmbiguous = (this.b$["JS.CIPChirality"].bsKekuleAmbiguous != null && this.b$["JS.CIPChirality"].bsKekuleAmbiguous.get (this.atomIndex));
+this.isDuplicate = this.multipleBondDuplicate = d;
+this.isKekuleAmbiguous = (this.b$["JS.CIPChirality"].data.bsKekuleAmbiguous != null && this.b$["JS.CIPChirality"].data.bsKekuleAmbiguous.get (this.atomIndex));
 this.elemNo = (d && this.isKekuleAmbiguous ? b.getKekuleElementNumber () : a.getElementNumber ());
 this.bondCount = a.getCovalentBondCount ();
 this.isSP3 = (this.bondCount == 4 || this.bondCount == 3 && !c && (this.elemNo > 10 || this.b$["JS.CIPChirality"].bsAzacyclic != null && this.b$["JS.CIPChirality"].bsAzacyclic.get (this.atomIndex)));
@@ -536,7 +490,6 @@ this.htPathPoints =  new java.util.Hashtable ();
 } else if (b != null) {
 this.htPathPoints = (b.htPathPoints).clone ();
 }this.bsPath = (b == null ?  new JU.BS () : JU.BSUtil.copy (b.bsPath));
-this.isDuplicate = this.multipleBondDuplicate = d;
 if (d) this.b$["JS.CIPChirality"].bsNeedRule.set (4);
 this.rootDistance = this.sphere;
 if (b == null) {
@@ -574,58 +527,63 @@ var c = 0;
 var d = 0;
 for (var e = a.length; --e >= 0; ) if ((b = a[e]).isCovalent ()) {
 var f = b.getOtherNode (this.atom);
-if (this.b$["JS.CIPChirality"].bsKekuleAmbiguous.get (f.getIndex ())) {
+if (this.b$["JS.CIPChirality"].data.bsKekuleAmbiguous.get (f.getIndex ())) {
 d++;
 c += f.getElementNumber ();
 }}
 return c / d;
 });
-Clazz.defineMethod (c$, "updateRingList", 
-function () {
-var a = JU.BSUtil.newAndSetBit (this.atomIndex);
-var b = this;
-var c = -1;
-while ((b = b.parent) != null && c != this.atomIndex) a.set (c = b.atomIndex);
-
-if (a.cardinality () <= 7) {
-for (var d = this.b$["JS.CIPChirality"].lstSmallRings.size (); --d >= 0; ) if (this.b$["JS.CIPChirality"].lstSmallRings.get (d).equals (a)) return;
-
-this.b$["JS.CIPChirality"].lstSmallRings.addLast (a);
-}});
 Clazz.defineMethod (c$, "setNode", 
 function () {
 if (this.isSet || (this.isSet = true) && this.isDuplicate) return true;
-var a = this.atom.getEdges ();
-var b = a.length;
+var a = this.atom.getIndex ();
+var b = this.atom.getEdges ();
+var c = b.length;
 if (JU.Logger.debuggingHigh) JU.Logger.info ("set " + this);
-var c = 0;
-for (var d = 0; d < b; d++) {
-var e = a[d];
-if (!e.isCovalent ()) continue;
-var f = e.getOtherNode (this.atom);
-var g = (this.parent != null && this.parent.atom === f);
-var h = e.getCovalentOrder ();
-if (h == 2) {
-if (this.elemNo > 10 || !this.b$["JS.CIPChirality"].isFirstRow (f)) h = 1;
+var d = 0;
+for (var e = 0; e < c; e++) {
+var f = b[e];
+if (!f.isCovalent ()) continue;
+var g = f.getOtherNode (this.atom);
+var h = (this.parent != null && this.parent.atom === g);
+var i = f.getCovalentOrder ();
+if (i == 2) {
+if (this.elemNo > 10 || !this.b$["JS.CIPChirality"].isFirstRow (g)) i = 1;
  else {
 this.isAlkene = true;
-if (g) this.setEne ();
-}}if (b == 1 && h == 1 && g) return this.isTerminal = true;
-switch (h) {
+if (h) this.setEne ();
+}}if (c == 1 && i == 1 && h) return this.isTerminal = true;
+switch (i) {
 case 3:
-if (this.addAtom (c++, f, g, false, g) == null) return !(this.isTerminal = true);
+if (this.addAtom (d++, g, h, false, h) == null) return !(this.isTerminal = true);
 case 2:
-if (this.addAtom (c++, f, h != 2 || g, h == 2, g) == null) return !(this.isTerminal = true);
+if (this.addAtom (d++, g, i != 2 || h, i == 2, h) == null) return !(this.isTerminal = true);
 case 1:
-if (g || this.addAtom (c++, f, h != 1 && this.elemNo <= 10, false, false) != null) break;
+if (h || this.addAtom (d++, g, i != 1 && this.elemNo <= 10, false, false) != null) break;
 default:
 return !(this.isTerminal = true);
 }
 }
-this.nAtoms = c;
-for (; c < this.atoms.length; c++) this.atoms[c] = Clazz.innerTypeInstance (JS.CIPChirality.CIPAtom, this, null).create (null, this, false, true, false);
+this.nAtoms = d;
+switch (d) {
+case 2:
+case 3:
+if (this.elemNo == 6 && this.b$["JS.CIPChirality"].data.bsCMinus.get (a) || this.b$["JS.CIPChirality"].data.bsXAromatic.get (a)) {
+this.nAtoms++;
+this.addAtom (d++, this.atom, true, false, false);
+}break;
+}
+if (d < 4) for (; d < this.atoms.length; d++) this.atoms[d] = Clazz.innerTypeInstance (JS.CIPChirality.CIPAtom, this, null).create (null, this, false, true, false);
 
+try {
 java.util.Arrays.sort (this.atoms);
+} catch (e) {
+if (Clazz.exceptionOf (e, Exception)) {
+e.printStackTrace ();
+} else {
+throw e;
+}
+}
 return true;
 });
 Clazz.defineMethod (c$, "setEne", 
@@ -1044,23 +1002,6 @@ Clazz.defineMethod (c$, "sign",
 function (a) {
 return (a < 0 ? -1 : a > 0 ? 1 : 0);
 }, "~N");
-Clazz.defineMethod (c$, "addSmallRings", 
-function (a) {
-if (this.atom == null || this.sphere > 7) return;
-if (a != null) a.clear (this.atom.getIndex ());
-if (this.isTerminal || this.isDuplicate || this.atom.getCovalentBondCount () > 4) return;
-var b;
-var c = 0;
-var d = this.atom.getEdges ();
-for (var e = d.length; --e >= 0; ) {
-var f = d[e];
-if (!f.isCovalent () || (b = f.getOtherNode (this.atom)).getCovalentBondCount () == 1 || this.parent != null && b === this.parent.atom) continue;
-var g = this.addAtom (c++, b, false, false, false);
-if (g.isDuplicate) g.updateRingList ();
-}
-for (var f = 0; f < c; f++) if (this.atoms[f] != null) this.atoms[f].addSmallRings (a);
-
-}, "JU.BS");
 Clazz.defineMethod (c$, "clone", 
 function () {
 var a = null;
@@ -1086,7 +1027,7 @@ return a;
 });
 Clazz.overrideMethod (c$, "toString", 
 function () {
-return (this.atom == null ? "<null>" : "[" + this.b$["JS.CIPChirality"].currentRule + "." + this.sphere + "," + this.id + "." + this.atom.getAtomName () + (this.isDuplicate ? "*(" + this.rootDistance + ")" : "") + (this.auxChirality == '~' ? "" : "" + this.auxChirality) + "]");
+return (this.atom == null ? "<null>" : "[" + this.b$["JS.CIPChirality"].currentRule + "." + this.sphere + "," + this.id + "." + (this.isDuplicate ? this.parent.atom : this.atom).getAtomName () + (this.isDuplicate ? "*(" + this.rootDistance + ")" : "") + (this.auxChirality == '~' ? "" : "" + this.auxChirality) + " " + this.elemNo + "]");
 });
 c$ = Clazz.p0p ();
 };
